/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id$
 *
 */

package org.ow2.sirocco.vmm.agent.bundle;

import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;
import org.osgi.framework.ServiceRegistration;
import org.osgi.util.tracker.ServiceTracker;
import org.ow2.sirocco.vmm.agent.main.VirtManagerAgent;
import org.ow2.sirocco.vmm.api.VMMAgentMXBean;
import org.ow2.sirocco.vmm.placementmanager.api.VMPlacementSolver;

public class Activator implements BundleActivator {
    private VirtManagerAgent agent;

    private ServiceRegistration registration;

    private VMPlacementSolverTracker vmPlacementSolverTracker;

    public void start(final BundleContext context) throws Exception {
        this.agent = VirtManagerAgent.getInstance();
        this.agent.start();
        this.registration = context.registerService(new String[] {VMMAgentMXBean.class.getName()}, this.agent, null);
        this.vmPlacementSolverTracker = new VMPlacementSolverTracker(context);
        this.vmPlacementSolverTracker.open();
    }

    public void stop(final BundleContext context) throws Exception {
        this.registration.unregister();
        this.agent.shutdown();
        this.vmPlacementSolverTracker.close();
    }

    private class VMPlacementSolverTracker extends ServiceTracker {

        public VMPlacementSolverTracker(final BundleContext bundleContext) {
            super(bundleContext, VMPlacementSolver.class.getName(), null);
        }

        @Override
        public Object addingService(final ServiceReference reference) {
            VMPlacementSolver manager = (VMPlacementSolver) this.context.getService(reference);
            Activator.this.agent.setVmPlacementSolver(manager);
            return manager;
        }

        @Override
        public void modifiedService(final ServiceReference sr, final Object o) {
            Activator.this.agent.setVmPlacementSolver(null);
            VMPlacementSolver manager = (VMPlacementSolver) this.context.getService(sr);
            Activator.this.agent.setVmPlacementSolver(manager);
        }

        @Override
        public void removedService(final ServiceReference reference, final Object service) {
            VMPlacementSolver manager = (VMPlacementSolver) service;
            // for clean up
            Activator.this.agent.setVmPlacementSolver(null);
            this.context.ungetService(reference);
        }

    }

}
